﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Xml;
using System.Xml.Xsl;
using System.Reflection;
using System.IO;

namespace Masteroppgave.Controller
{
    public class Logic
    {
        private static TreeView _treeViewLeft;
        private static TreeView _treeViewRight;
        private static ImageList _imageList;

        public static TreeView TreeViewLeft
        {
            set
            {
                SaveToModel();
            }
            get
            {
                if (_treeViewLeft == null)
                {
                    _treeViewLeft = Model.Tools.LoadTree();
                }
                return _treeViewLeft;
            }
        }

        public static TreeView TreeViewRight
        {
            set
            {
                SaveToModel();
            }
            get
            {
                if (_treeViewRight == null)
                    _treeViewRight = new TreeView();
                return _treeViewRight;
            }
        }

        public static ImageList ImageList
        {
            get
            {
                if (_imageList == null)
                {
                    _imageList = new ImageList();
                    string gfxpath = "Masteroppgave.Model.gfx.";
                    _imageList.Images.Add("i", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "i.gif")));
                    _imageList.Images.Add("f", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "f.gif")));
                    _imageList.Images.Add("b", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "b.gif")));
                    _imageList.Images.Add("a", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "a.gif")));
                    _imageList.Images.Add("t", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "t.gif")));
                    _imageList.Images.Add("r", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "r.gif")));
                    _imageList.Images.Add("accept", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "accept.ico")));
                    _imageList.Images.Add("cancel", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "cancel.png")));
                    _imageList.Images.Add("edit", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "edit.ico")));
                    _imageList.Images.Add("delete", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "delete.ico")));
                    _imageList.Images.Add("add", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "add.ico")));
                    _imageList.Images.Add("forward", new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(gfxpath + "forward.png")));

                    TreeViewRight.ImageList = _imageList;
                    TreeViewLeft.ImageList = _imageList;

                }
                return _imageList;
            }
        }

        public static void SaveToModel()
        {
            if(_treeViewLeft != null && _treeViewRight != null)
                Model.Tools.SaveTree(_treeViewLeft, _treeViewRight);

        }

        /// <summary>
        /// Adds an empty Input item to the input treeview
        /// </summary>
        /// <param name="tv"></param>
        private static void AddInputItem(TreeView tv)
        {
            Model.InputItem i = new Model.InputItem();
            
            // If adding the item to the right tree 
            if (tv == _treeViewRight)
                // Then it should be checked
                i.Checked = true;
            // The left tree
            else
                // Not checked
                i.Checked = false;

            tv.Nodes.Add(i);
            tv.SelectedNode = i;

            SaveToModel();
        }

        /// <summary>
        /// Adds an empty Input item to the left treeview
        /// </summary>
        public static void AddInputItemLeftTree()
        {
            AddInputItem(_treeViewLeft);
        }

        /// <summary>
        /// Adds an empty Input item to the right treeview
        /// </summary>
        public static void AddInputItemRightTree()
        {
            AddInputItem(_treeViewRight);
        }

        public static void AddChildItem(Model.NodeModel parent)
        {
            // If parent aint null
            if (parent != null)
            {
                // Create a new, empty child
                Model.NodeModel node = parent.AddEmptyChild();
                
                // Update it..
                parent.UpdateNode();
                parent.Expand();

                // If the new node aint null
                if (node != null)
                {
                    // Set the selected node to the new node
                    node.TreeView.SelectedNode = node;
                    
                }
            }
        }

        public static void DeleteItem(Model.NodeModel node)
        {
            if (node != null)
            {
                Model.NodeModel parent = (Model.NodeModel)node.Parent;
                
                // If node has a parent, the use the RemoveChild method and set focus to the parent
                if (parent != null)
                {
                    parent.RemoveChild(node);
                    parent.TreeView.SelectedNode = parent;
                }
                // Else just remove the input item from the treeview
                else node.Remove();

                // Save the changes
                SaveToModel();
                
            }
        }

        #region --Results--

        private static XmlDocument xDoc;

        private static string
            htmLoc = Application.StartupPath + "\\Result.htm",
            xmlLoc = Application.StartupPath + "\\treeView.xml";

        /// <summary>
        /// Prints the results of the rightmost list to a html file and shows it in a browser
        /// </summary>
        public static void PrintResults()
        {
            // Save gfx to disc if not there

            // Creates gfx directory if it doesnt exist
            #region -- Create gfx files --

            Stream i, f, b, a, t;
            string strI, strF, strB, strA, strT, strgfx;
            i = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("Masteroppgave.Model.gfx.i.gif");
            f = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("Masteroppgave.Model.gfx.f.gif");
            b = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("Masteroppgave.Model.gfx.b.gif");
            a = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("Masteroppgave.Model.gfx.a.gif");
            t = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("Masteroppgave.Model.gfx.t.gif");

            strgfx = Application.StartupPath + "\\gfx\\";

            strI = strgfx + "i.gif";
            strF = strgfx + "f.gif";
            strB = strgfx + "b.gif";
            strA = strgfx + "a.gif";
            strT = strgfx + "t.gif";

            Stream iStream, fStream, bStream, aStream, tStream;

            if (!Directory.Exists(strgfx))
            {
                Directory.CreateDirectory(strgfx);
                
            }

            if (!File.Exists(strI))
            {
                iStream = File.Create(strI);
                BinaryReader br = new BinaryReader(i);

                BinaryWriter bw = new BinaryWriter(iStream);

                bw.Write(br.ReadBytes((int)i.Length));

                bw.Flush();

                //clean up 
                bw.Close();
                br.Close();
                i.Close();
            }

            if (!File.Exists(strA))
            {
                aStream = File.Create(strA);
                BinaryReader br = new BinaryReader(a);

                BinaryWriter bw = new BinaryWriter(aStream);

                bw.Write(br.ReadBytes((int)a.Length));

                bw.Flush();

                //clean up 
                bw.Close();
                br.Close();
                a.Close();
            }

            if (!File.Exists(strF))
            {
                fStream = File.Create(strF);
                BinaryReader br = new BinaryReader(f);

                BinaryWriter bw = new BinaryWriter(fStream);

                bw.Write(br.ReadBytes((int)f.Length));

                bw.Flush();

                //clean up 
                bw.Close();
                br.Close();
                f.Close();
            }

            if (!File.Exists(strB))
            {
                bStream = File.Create(strB);
                BinaryReader br = new BinaryReader(b);

                BinaryWriter bw = new BinaryWriter(bStream);

                bw.Write(br.ReadBytes((int)b.Length));

                bw.Flush();

                //clean up 
                bw.Close();
                br.Close();
                b.Close();
            }

            if (!File.Exists(strT))
            {
                tStream = File.Create(strT);
                BinaryReader br = new BinaryReader(t);

                BinaryWriter bw = new BinaryWriter(tStream);

                bw.Write(br.ReadBytes((int)t.Length));

                bw.Flush();

                //clean up 
                bw.Close();
                br.Close();
                t.Close();
            }




            #endregion

            xDoc = new XmlDocument();

            XmlElement root = xDoc.CreateElement("treenodes");

            xDoc.AppendChild(root);

            ConvertToXml(root, Logic.TreeViewRight.Nodes);

            xDoc.Save(xmlLoc);

            TransformXml();

        }



        private static void ConvertToXml(XmlNode parent, TreeNodeCollection nodes)
        {

            foreach (Model.NodeModel treeNode in nodes)
            {

                XmlElement node = xDoc.CreateElement("node");

                XmlAttribute text = xDoc.CreateAttribute("text");

                XmlAttribute descr = xDoc.CreateAttribute("descr");

                XmlAttribute type = xDoc.CreateAttribute("type");

                text.Value = treeNode.Text;

                descr.Value = treeNode.Description;

                type.Value = treeNode.Type;

                node.Attributes.Append(text);
                node.Attributes.Append(descr);
                node.Attributes.Append(type);

                ConvertToXml(node, treeNode.Nodes);

                parent.AppendChild(node);

            }

        }



        private static void TransformXml()
        {

            //Converts the XmlDocument into HTML

            XslCompiledTransform xslt = new XslCompiledTransform();

            xslt.Load(new XmlTextReader(Assembly.GetExecutingAssembly().GetManifestResourceStream("Masteroppgave.Model.files.treeView.xslt")));

            xslt.Transform(xmlLoc, htmLoc);

            File.Delete(xmlLoc);

            //Help.ShowHelp(null, htmLoc);

            System.Diagnostics.Process.Start(htmLoc);

        }

        #endregion
    }
}
