using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Windows.Forms;
using Masteroppgave.Model;

namespace Masteroppgave.Model
{
	public class Tools
	{
	    static readonly string filename = "Database.osl";
		
        	
        /// <summary>
        /// Serializes the InputItems in treeViewRight and treeViewLeft
        /// </summary>
        /// <param name="tree1">treeViewLeft</param>
        /// <param name="tree2">treeViewRight</param>
        public static void SaveTree(TreeView tree1, TreeView tree2)
		{
            
			Stream file = File.Open(filename, FileMode.Create);
            
			BinaryFormatter bf = new BinaryFormatter();
			try
			{
				// Serialize file
                foreach (InputItem tn in tree1.Nodes)
                {
                    bf.Serialize(file, tn);
                }

                foreach (InputItem tn in tree2.Nodes)
                {
                    bf.Serialize(file, tn);
                }


			}
			catch (System.Runtime.Serialization.SerializationException e)
			{
                MessageBox.Show("Serialization failed : " + e.Message);
			}
            finally { file.Close(); }
		}

	    /// <summary>
	    /// Loads a the nodes from the serialized file, puts them in a TreeView and returns it
	    /// </summary>
	    /// <returns>A TreeView containing the de-serialized nodes</returns>
		public static TreeView LoadTree()
		{
		    Stream file;
            TreeView tmpTree = new TreeView();
            // If file exists
            if (File.Exists(filename))
            {
                // Open file
                 file = File.Open(filename, FileMode.Open);
            }
            // If the file could not be found
            else
            {
                // Load the file from the embedded resource database
                DialogResult d = MessageBox.Show("The database file could not be found. If this is the first time you use the program this is normal and a database file will be created for you. If else you must look up the file ("+ filename+") and put it in the same folder as your program." + Environment.NewLine + Environment.NewLine + "To proceed with the creation of the new database file, press OK", "File not found", MessageBoxButtons.OK, MessageBoxIcon.Information);

                file = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("Masteroppgave.Model.files."+filename);

                Stream ToStream = File.Create(filename);

                if(d == DialogResult.OK)
                {
                    // Save a new file from embedded resource

                    BinaryReader br = new BinaryReader(file);
                    
                    BinaryWriter bw = new BinaryWriter(ToStream);

                    bw.Write(br.ReadBytes((int)file.Length));

                    bw.Flush();

                    //clean up 
                    bw.Close();
                    br.Close();
                    file.Close();
                    return LoadTree();
                }
              
            }


            BinaryFormatter bf = new BinaryFormatter();
            try
            {
               
                while (file.Position != file.Length)
                    // Add nodes to the tree
                    tmpTree.Nodes.Add((NodeModel)bf.Deserialize(file));

                tmpTree.Sort();
                return tmpTree;
            }
            catch (Exception e)
            {
                throw e;
            }
            finally
            {
                // Close file
                file.Close();
            }
		}
		
	}
}