using System;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using Masteroppgave.Model;
using Masteroppgave.Controller;

namespace Masteroppgave.View
{
    public partial class frmMain : Form
    {
        #region --Parameters
        NodeModel _selectedNode = null;

        readonly ContextMenu rightRootMenu, rightIMenu, rightFMenu, rightBMenu, rightAMenu, rightTMenu, leftRootMenu, leftIMenu;
        #endregion

        #region --Constructors--
        /// <summary>
        /// Main constructor
        /// </summary>
        public frmMain()
        {
        
            InitializeComponent();

            #region --Menus--
            // Menu
            rightRootMenu = new ContextMenu();
            rightIMenu = new ContextMenu();
            rightFMenu = new ContextMenu();
            rightBMenu = new ContextMenu();
            rightAMenu = new ContextMenu();
            rightTMenu = new ContextMenu();
            leftRootMenu = new ContextMenu();
            leftIMenu = new ContextMenu();

            // Left menu items
            MenuItem menuItemLeftAddInputItem = new MenuItem("Add Input item", EventAddInputItemLeftTree);
            MenuItem menuItemLeftEditItem = new MenuItem("Edit item", EventEditItem);
            MenuItem menuItemLeftDeleteItem = new MenuItem("Delete item", EventDeleteItem);
            MenuItem menuItemLeftMoveItem = new MenuItem("Move item to Selection tree >>", EventMoveItem);

            // Right menu items
            MenuItem menuItemRightAddI = new MenuItem("Add Input item", EventAddInputItemRightTree);
            MenuItem menuItemRightAddF = new MenuItem("Add Failure mode", EventAddNewChild);
            MenuItem menuItemRightAddB = new MenuItem("Add Barrier", EventAddNewChild);
            MenuItem menuItemRightAddA = new MenuItem("Add Action", EventAddNewChild);
            MenuItem menuItemRightAddT = new MenuItem("Add Test", EventAddNewChild);
            MenuItem menuItemRightEditItem = new MenuItem("Edit item", EventEditItem);
            MenuItem menuItemRightDeleteItem = new MenuItem("Delete item", EventDeleteItem);

            // Left Root menu
            leftRootMenu.MenuItems.Add(menuItemLeftAddInputItem);

            // Left Input item menu
            leftIMenu.MenuItems.Add(menuItemLeftAddInputItem.CloneMenu());
            leftIMenu.MenuItems.Add(menuItemLeftMoveItem);
            leftIMenu.MenuItems.Add(menuItemLeftEditItem);
            leftIMenu.MenuItems.Add(menuItemLeftDeleteItem);

            // Right Root menu
            rightRootMenu.MenuItems.Add(menuItemRightAddI);

            // Specified add items for each item
            rightIMenu.MenuItems.Add(menuItemRightAddF);
            rightIMenu.MenuItems.Add(menuItemRightEditItem.CloneMenu());
            rightIMenu.MenuItems.Add(menuItemRightDeleteItem.CloneMenu());

            rightFMenu.MenuItems.Add(menuItemRightAddB);
            rightFMenu.MenuItems.Add(menuItemRightEditItem.CloneMenu());
            rightFMenu.MenuItems.Add(menuItemRightDeleteItem.CloneMenu());
            
            rightBMenu.MenuItems.Add(menuItemRightAddA);
            rightBMenu.MenuItems.Add(menuItemRightEditItem.CloneMenu());
            rightBMenu.MenuItems.Add(menuItemRightDeleteItem.CloneMenu());
            
            rightAMenu.MenuItems.Add(menuItemRightAddT);
            rightAMenu.MenuItems.Add(menuItemRightEditItem.CloneMenu());
            rightAMenu.MenuItems.Add(menuItemRightDeleteItem.CloneMenu());

            rightTMenu.MenuItems.Add(menuItemRightEditItem.CloneMenu());
            rightTMenu.MenuItems.Add(menuItemRightDeleteItem.CloneMenu());

            #endregion
            
            #region --Images--

            // Sets the images according to the ImageList in the Logic
            btnAdd.Image = Logic.ImageList.Images["add"];
            btnCancelChange.Image = Logic.ImageList.Images["cancel"];
            btnDelete.Image = Logic.ImageList.Images["delete"];
            btnSave.Image = Logic.ImageList.Images["accept"];
            btnAddInputItem.Image = Logic.ImageList.Images["i"];

            lblPic1.Image = Logic.ImageList.Images[0];
            lblPic1.Text = "  ";
            lblPic2.Image = Logic.ImageList.Images[0];
            lblPic2.Text = "  ";

            lblInputItem.Image = Logic.ImageList.Images["i"];
            lblFailureMode.Image = Logic.ImageList.Images["f"];
            lblBarrier.Image = Logic.ImageList.Images["b"];
            lblAction.Image = Logic.ImageList.Images["a"];
            lblTest.Image = Logic.ImageList.Images["t"];

            #endregion
 
            // The model that is described in the bottom left of the main form
            lblInputItem.Text = "      " + "Input item";
            lblFailureMode.Text = "      " + "Failure mode";
            lblBarrier.Text = "      " + "Barrier";
            lblAction.Text = "      " + "Action";
            lblTest.Text = "      " + "Test";
           
        }

        #endregion

        #region --Properties--

        /// <summary>
        /// The node that was last selected and that you can perform tasks on, like for instance deleting, adding
        /// a child or edit
        /// </summary>
        private NodeModel SelectedNode
        {
            get { return _selectedNode; }
            set 
            {
                if (value != null)
                {
                    SetNodeInformation(value);
                }
                else
                {
                    btnMoveLeft.Visible = false;
                    btnMoveRight.Visible = false;
                }
                _selectedNode = value;
            }
        }
        
        #endregion

        #region --Methods--

        /// <summary>
        /// Used to check if the there are edited data which are not saved yet 
        /// </summary>
        private void CheckForUnsavedData()
        {
            // If the button is enabled then the selected node is edited
            if (btnSave.Enabled)
            {
                DialogResult d = MessageBox.Show("Do you want to save the changes to " + SelectedNode.Name + "?", "Save changes", MessageBoxButtons.YesNo);
                if (d == DialogResult.Yes)
                {
                    SetSaveButton(false);
                    NodeModel nm = (NodeModel)SelectedNode;
                    nm.Name = txtItemName.Text;
                    nm.Description = txtItemDescription.Text;
                    SaveData();

                }
                else if (d == DialogResult.No)
                    EventBtnCancelChange_Click(null, null);
            }

        }

        /// <summary>
        /// Ask the user if he wants to delete the node, and if so tells the Logic to delete
        /// </summary>
        /// <param name="node">The node that should be deleted</param>
        private void DeleteItem(NodeModel node)
        {
            try
            {
                if (node == null) return;

                // Asks the user if he wants to delete
                DialogResult d = MessageBox.Show("Are you sure you want to delete " + node.Name + " and all underlying nodes? This cannot be undone!", "Confirm delete", MessageBoxButtons.OKCancel, MessageBoxIcon.Stop, MessageBoxDefaultButton.Button2);

                // If Yes
                if (d == DialogResult.OK)
                {

                    TreeView tree = node.TreeView;

                    // Tells the Logic to delete
                    Logic.DeleteItem(node);

                    // Updates move buttons
                    SetMoveButtons(tree);
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());

            }
        }

        /// <summary>
        /// Stores/serializes the nodes in the treeviews
        /// </summary>
        private void SaveData()
        {
            Logic.SaveToModel();

            // Somewhat dirty implementation 
            SelectedNode.ToolTipText = txtItemDescription.Text;
        }
        
        /// <summary>
        /// Sets the buttons beneath the lists (used to move nodes between lists) 
        /// to visible or not 
        /// </summary>
        /// <param name="tree">The selected tree</param>
        private void SetMoveButtons(TreeView tree)
        {
            
            if(tree == Logic.TreeViewLeft)
            {
                Logic.TreeViewRight.SelectedNode = null;
                btnMoveLeft.Visible = false;

                if (tree.SelectedNode != null)
                {
                    if (!btnMoveRight.Visible)
                        btnMoveRight.Visible = true;
                }
                else
                    btnMoveRight.Visible = false;

            }
            else if(tree == Logic.TreeViewRight)
            {
                Logic.TreeViewLeft.SelectedNode = null;
                btnMoveRight.Visible = false;

                if(tree.SelectedNode is InputItem)
                    btnMoveLeft.Visible = true;
                else
                    btnMoveLeft.Visible = false;
            }


        }

        /// <summary>
        /// Used to configure the Save changes button 
        /// </summary>
        /// <param name="enabled">Whether the button should be enabled or disabled</param>
        private void SetSaveButton(bool enabled)
        {
            if (enabled)
            {
                btnSave.Enabled = true;
                btnSave.Text = "Save";
            }
            else
            {
                btnSave.Enabled = false;
                btnSave.Text = "Saved";
            }
        }

        /// <summary>
        /// Used to set details in the Information groupbox when a new node is selected.
        /// </summary>
        /// <param name="nm"></param>
        private void SetNodeInformation(NodeModel nm)
        {
            // If there is a selected node
            // It is always a NodeModel due to the parameter in the method. This method is never called for a root node.
            if (nm != null)
            {
                CheckForUnsavedData();

                SetTextboxes(true);

                txtItemName.Text = nm.Name;
                txtItemDescription.Text = nm.Description;

                lblPic1.Image = Logic.TreeViewRight.ImageList.Images[nm.ImageIndex];
                lblPic2.Image = Logic.TreeViewRight.ImageList.Images[nm.ImageIndex];
                
                SetSaveButton(false);
                btnSave.Text = "Save";
            }
        }

        /// <summary>
        /// Sets the textboxes and cancel button to the bool value "enabled". 
        /// If enabled = false the textboxes are cleared and diabled.
        /// </summary>
        /// <param name="enabled"></param>
        private void SetTextboxes(bool enabled)
        {
            // Enables the textboxes and buttons
            if (enabled)
            {
                txtItemName.Enabled = true;
                txtItemDescription.Enabled = true;
                btnCancelChange.Enabled = true;
            }
            // Disables the textboxes and buttons
            else
            {
                txtItemDescription.Clear();
                txtItemName.Clear();
                txtItemDescription.Enabled = false;
                txtItemName.Enabled = false;
                //btnSave.Enabled = false;
                btnCancelChange.Enabled = false;

                lblPic1.Image = null;
                lblPic2.Image = null;
            }
        }

        /// <summary>
        /// Enables or disables the menu button Add, Edit and Delete item
        /// </summary>
        /// <param name="add">Enable or disable Add button</param>
        /// <param name="edit">Enable or disable Edit button</param>
        /// <param name="delete">Enable or disable Delete button</param>
        private void SetMenuButtons(bool add, bool edit, bool delete)
        {
            btnAdd.Enabled = add;
            btnDelete.Enabled = delete;

            // If the add button is enabled
            if (add)
                btnAdd.Text = "       Add " + SelectedNode.ChildType;
            
            // If the add button is disabled
            else
                btnAdd.Text = "       Add item";

        }

        #endregion

        #region --Events--

        #region ---Buttons---

        /// <summary>
        /// Save changes made to a item
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventSaveItemChanges(object sender, EventArgs e)
        {
            SetSaveButton(false);
            NodeModel nm = (NodeModel)SelectedNode;
            nm.Name = txtItemName.Text;
            nm.Description = txtItemDescription.Text;
            nm.TreeView.SelectedNode = nm;
            btnSave.Text = "Saved";

            SaveData();
        }

        private void EventBtnCancelChange_Click(object sender, EventArgs e)
        {
            NodeModel nm = (NodeModel)SelectedNode;
            txtItemName.Text = nm.Name;
            txtItemDescription.Text = nm.Description;
        }
   
        private void EventBtnDelete_Click(object sender, EventArgs e)
        {
            DeleteItem(SelectedNode);
        }

        private void EventBtnQuit_Click(object sender, EventArgs e)
        {
            DialogResult d = MessageBox.Show("Are you sure you want to quit?", "You're done?", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
            if (d == DialogResult.Yes)
                Dispose();
        }

        /// <summary>
        /// When the Finish button is clicked
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventBtnFinish_Click(object sender, EventArgs e)
        {
            Logic.PrintResults();

            MessageBox.Show(
                "A html file has been created with your results, and it should pop up right now. If it doesnt, look for 'Result.htm' next to your program and view it.",
                "Results", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        #endregion

        #region ---TreeViews---

        #region ---Drag and drop---
        
        /// <summary>
        /// When a node is dropped in a tree
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTree_DragDrop(object sender, DragEventArgs e)
        {
            try
            {
                // Get the tree.
                TreeView tree = (TreeView)sender;

                InputItem i = (InputItem)e.Data.GetData(typeof(InputItem));
                i.Remove();
                tree.Nodes.Add(i);
                if (tree == Logic.TreeViewRight)
                {
                    i.Checked = true;
                    i.ShowChildren(true);
                }
                else
                {
                    i.Checked = false;
                    i.ShowChildren(false);
                }

                i.Expand();

            }
            catch (Exception ex)
            { MessageBox.Show(ex.ToString()); }
        }

        /// <summary>
        /// Occurs when an item is dragged over a tree
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTree_DragOver(object sender, DragEventArgs e)
        {
            // Get the tree.
            TreeView tree = (TreeView)sender;

            // Drag and drop denied by default.
            e.Effect = DragDropEffects.None;

            InputItem i = (InputItem)e.Data.GetData(typeof(InputItem));

            // Is it a valid format and is the target tree different than the source tree?
            if (i != null && i.TreeView != tree)
            {
                e.Effect = DragDropEffects.Move;
            }
        }

        /// <summary>
        /// When a mouse button is clicked
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTree_MouseDown(object sender, MouseEventArgs e)
        {
            // Kun drag and drop for left mouse button
            if (e.Button != MouseButtons.Left) return;

            // Get the tree.
            TreeView tree = (TreeView)sender;

            // Get the node underneath the mouse.
            TreeNode node = tree.GetNodeAt(e.X, e.Y);
            tree.SelectedNode = node;

            // Start the drag-and-drop operation with a cloned copy of the node.
            if (node != null && node is InputItem)
            {
                tree.DoDragDrop(node, DragDropEffects.Move);
            }

        }

        #endregion

        #region ---Other---
       
        /// <summary>
        /// When a key is pushed and released
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTree_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                EventDeleteItem(sender, e);
            }
        }

        #endregion
        
        #region ---TreeViewRight---

        /// <summary>
        /// Occurs after a node has been checked. If the node is unchecked the underlying nodes are unchecked,
        /// if the node is checked underlying nodes are checked
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTreeViewRight_AfterCheck(object sender, TreeViewEventArgs e)
        {
            // If the checked node is a NodeModel
            if (e.Node is NodeModel)
            {
                NodeModel node = (NodeModel)e.Node;
                if (node.Checked)
                {
                    // When a node is checked then all parents will be checked if they are unchecked
                    if(node.Parent != null)
                        if(!node.Parent.Checked) node.Parent.Checked = true;

                    // If node is Test
                    if (node is Test)
                        // Then do nothing more
                        return;
                    else
                    {
                        // Checks if one of the childs are checked, if so the rest of the childs should not be checked
                        bool check = false;
                        foreach (NodeModel child in node.Nodes)
                        {
                            check = check || child.Checked;
                        }

                        // If none of the childs are checked
                        if(!check)
                            // Then check every child
                            ((NodeModel)e.Node).CheckChildren(true);
                    }
                }
                else
                {
                    // Uncheck all children
                    ((NodeModel)e.Node).CheckChildren(false);
                    
                    // If it is an input item then move it to the input item list
                    if (e.Node is InputItem)
                    {
                        e.Node.Remove();
                        //e.Node.Parent.Nodes.Remove(e.Node);
                        e.Node.Checked = false;
                        Logic.TreeViewLeft.Nodes.Add(e.Node);
                        e.Node.Nodes.Clear();
                    }
                }
            }
            // It is the root node
            else
            {
                // Should always be checked
                if (!e.Node.Checked)
                    e.Node.Checked = true;

            }
        }

        /// <summary>
        /// Occurs after the a node is selected in Logic.TreeViewRight. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTreeViewRight_AfterSelect(object sender, TreeViewEventArgs e)
        {
            SetMoveButtons(Logic.TreeViewRight);

            SelectedNode = (NodeModel)Logic.TreeViewRight.SelectedNode;

            // Selected node is NodeModel
            if (Logic.TreeViewRight.SelectedNode is NodeModel)
            {
                if (SelectedNode is Test)
                    SetMenuButtons(false, true, true);
             
                else if(SelectedNode is InputItem)
                    SetMenuButtons(true, true, true);
                
                else
                    SetMenuButtons(true, true, true);
            }
        }

        /// <summary>
        /// Shows the menu upon mouse rightclick
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTreeViewRight_MouseUp(object sender, MouseEventArgs e)
        {
         
            // If it is a rightclick
            if (e.Button == MouseButtons.Right)
            {
                // Get clicked node
                TreeNode node = Logic.TreeViewRight.GetNodeAt(new Point(e.X, e.Y));
                Logic.TreeViewRight.SelectedNode = node;
                
                // Show input item menu
                if (node is InputItem)
                    rightIMenu.Show(Logic.TreeViewRight, new Point(e.X, e.Y));
                // Show failure mode menu
                else if (node is FailureMode)
                    rightFMenu.Show(Logic.TreeViewRight, new Point(e.X, e.Y));
                // Show barrier menu
                else if (node is Barrier)
                    rightBMenu.Show(Logic.TreeViewRight, new Point(e.X, e.Y));
                // Show action menu
                else if (node is Action)
                    rightAMenu.Show(Logic.TreeViewRight, new Point(e.X, e.Y));
                // Show test menu
                else if (node is Test)
                    rightTMenu.Show(Logic.TreeViewRight, new Point(e.X, e.Y));
                // Show Root menu
                else
                    rightRootMenu.Show(Logic.TreeViewRight, new Point(e.X, e.Y));
            }
        }

        #endregion

        #region ---TreeViewLeft---

        /// <summary>
        /// After a node is checked in the left treeview
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTreeViewLeft_AfterCheck(object sender, TreeViewEventArgs e)
        {
            // If the checked node is a NodeModel
            if (e.Node is NodeModel)
            {
                NodeModel nm = (NodeModel)e.Node;
                // If it is checked
                if (nm.Checked)
                {
                    Logic.TreeViewLeft.Nodes.Remove(nm);
                    Logic.TreeViewRight.Nodes.Add(nm);
                    nm.ShowChildren(true);
                    nm.Expand();
                }
            }
        }

        /// <summary>
        /// Occurs when a node in the Input items tree is selected
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTreeViewLeft_AfterSelect(object sender, TreeViewEventArgs e)
        {
            SetMoveButtons(Logic.TreeViewLeft);
            
            SelectedNode = (NodeModel)Logic.TreeViewLeft.SelectedNode;
            
            // If the selected node is an Input item
            if (Logic.TreeViewLeft.SelectedNode is InputItem)
            {
                SetMenuButtons(false, true, true);
                //btnMoveRight.Visible = true;
            }
        }

        /// <summary>
        /// Shows the menu in the left menu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTreeViewLeft_MouseUp(object sender, MouseEventArgs e)
        {
            // If right click
            if (e.Button == MouseButtons.Right)
            {
                // Get clicked node
                TreeNode node = Logic.TreeViewLeft.GetNodeAt(new Point(e.X, e.Y));
                Logic.TreeViewLeft.SelectedNode = node;

                // Shows the item menu
                if (node is NodeModel)
                    leftIMenu.Show(Logic.TreeViewLeft, new Point(e.X, e.Y));
                else
                    leftRootMenu.Show(Logic.TreeViewLeft, new Point(e.X, e.Y));


            }
        }

        /// <summary>
        /// Doubleclick is the same as check, moves the item to the right list
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTreeViewLeft_DoubleClick(object sender, EventArgs e)
        {
            if (Logic.TreeViewLeft.SelectedNode is NodeModel)
                Logic.TreeViewLeft.SelectedNode.Checked = true;
        }
        
        #endregion
     
        #endregion

        #region ---Textboxes---

        /// <summary>
        /// Event that is thrown when Enter is pushed while in txt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventTxtItemName_KeyUp(object sender, KeyEventArgs e)
        {
            // If enter is pushed
            if (e.KeyCode == Keys.Enter)
                // Save data
                EventSaveItemChanges(sender, null);
        }

        private void EventTxtItemName_TextChanged(object sender, EventArgs e)
        {
            if (!btnSave.Enabled && !txtItemDescription.Text.Equals(""))
                SetSaveButton(true);
        }

        private void EventTxtItemDescription_TextChanged(object sender, EventArgs e)
        {
            
            if (!btnSave.Enabled && !txtItemDescription.Text.Equals(""))
                SetSaveButton(true);
        }

        #endregion

        #region ---Other---
        
        private void frmMain_Load(object sender, EventArgs e)
        {
            btnMoveRight.Visible = false;
            btnMoveLeft.Visible = false;

            // 
            // treeViewRight
            // 
            Logic.TreeViewRight.AllowDrop = true;
            Logic.TreeViewRight.CheckBoxes = true;
            Logic.TreeViewRight.HideSelection = false;
            Logic.TreeViewRight.Location = new System.Drawing.Point(473, 56);
            Logic.TreeViewRight.Name = "treeViewRight";
            Logic.TreeViewRight.RightToLeft = System.Windows.Forms.RightToLeft.No;
            Logic.TreeViewRight.ShowNodeToolTips = true;
            Logic.TreeViewRight.Size = new System.Drawing.Size(257, 590);
            Logic.TreeViewRight.TabIndex = 21;
            this.toolTip1.SetToolTip(Logic.TreeViewRight, "This is the list of all the input items that you have chosen to be relevant for y" +
                    "our program. Here you can specify which failure modes etc which you want to incl" +
                    "ude in your program");
            Logic.TreeViewRight.AfterCheck += new System.Windows.Forms.TreeViewEventHandler(this.EventTreeViewRight_AfterCheck);
            Logic.TreeViewRight.MouseUp += new System.Windows.Forms.MouseEventHandler(this.EventTreeViewRight_MouseUp);
            Logic.TreeViewRight.DragDrop += new System.Windows.Forms.DragEventHandler(this.EventTree_DragDrop);
            Logic.TreeViewRight.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.EventTreeViewRight_AfterSelect);
            Logic.TreeViewRight.MouseDown += new System.Windows.Forms.MouseEventHandler(this.EventTree_MouseDown);
            Logic.TreeViewRight.KeyUp += new System.Windows.Forms.KeyEventHandler(this.EventTree_KeyUp);
            Logic.TreeViewRight.DragOver += new System.Windows.Forms.DragEventHandler(this.EventTree_DragOver);
            // 
            // treeViewLeft
            // 
            Logic.TreeViewLeft.AllowDrop = true;
            Logic.TreeViewLeft.CheckBoxes = true;
            Logic.TreeViewLeft.HideSelection = false;
            Logic.TreeViewLeft.Location = new System.Drawing.Point(234, 55);
            Logic.TreeViewLeft.Name = "treeViewLeft";
            Logic.TreeViewLeft.RightToLeft = System.Windows.Forms.RightToLeft.No;
            Logic.TreeViewLeft.ShowNodeToolTips = true;
            Logic.TreeViewLeft.Size = new System.Drawing.Size(193, 590);
            Logic.TreeViewLeft.TabIndex = 39;
            this.toolTip1.SetToolTip(Logic.TreeViewLeft, "From this list of input items you choose which input items that suit your program" +
                    ". Check, doubleclick  or use the button below to add it to the rightmost list");
            Logic.TreeViewLeft.AfterCheck += new System.Windows.Forms.TreeViewEventHandler(this.EventTreeViewLeft_AfterCheck);
            Logic.TreeViewLeft.DoubleClick += new System.EventHandler(this.EventTreeViewLeft_DoubleClick);
            Logic.TreeViewLeft.DragDrop += new System.Windows.Forms.DragEventHandler(this.EventTree_DragDrop);
            Logic.TreeViewLeft.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.EventTreeViewLeft_AfterSelect);
            Logic.TreeViewLeft.MouseDown += new System.Windows.Forms.MouseEventHandler(this.EventTree_MouseDown);
            Logic.TreeViewLeft.MouseUp += new System.Windows.Forms.MouseEventHandler(this.EventTreeViewLeft_MouseUp);
            Logic.TreeViewLeft.KeyUp += new System.Windows.Forms.KeyEventHandler(this.EventTree_KeyUp);
            Logic.TreeViewLeft.DragOver += new System.Windows.Forms.DragEventHandler(this.EventTree_DragOver);


            Controls.Add(Logic.TreeViewLeft);
            Controls.Add(Logic.TreeViewRight);

        }

        /// <summary>
        /// Event that throws the order of deleting the selected node
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventDeleteItem(object sender, EventArgs e)
        {
            DeleteItem(SelectedNode);
        }

        /// <summary>
        /// Tells the Logic to add a new input item
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventAddInputItemLeftTree(object sender, EventArgs e)
        {
            Logic.AddInputItemLeftTree();
        }

        /// <summary>
        /// Tells the Logic to add a new input item
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventAddInputItemRightTree(object sender, EventArgs e)
        {
            Logic.AddInputItemRightTree();
        }

        /// <summary>
        /// Tells the Logic to add a new child node to the selected node
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventAddNewChild(object sender, EventArgs e)
        {
            Logic.AddChildItem(SelectedNode);
        }

        /// <summary>
        /// Moves the item from a list to another
        /// </summary>
        private void EventMoveItem(object sender, EventArgs e)
        {
            if (SelectedNode != null && SelectedNode is InputItem)
            {
                // Moves the node to the other list
                TreeView tv = SelectedNode.TreeView;
                SelectedNode.Checked = !SelectedNode.Checked;

                // If the node that was moved was the last in the list
                if (tv.Nodes.Count == 0)
                {
                    // Selected node is set to null since we dont want it to be selected in the new list
                    SelectedNode = null;
                }
            }
        }

        /// <summary>
        /// Sets focus to the Name
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EventEditItem(object sender, EventArgs e)
        {
            groupBoxItemInformation.Focus();
            txtItemName.Focus();
        }
 
        #endregion

        #endregion
     
    }
}
